// EPROM Generation program for 405 line test card signal
// V1.0 Copyright (c) 2000-2002 J. Evans
// This software is supplied as-is and with no warrenty expressed or implied.
// Feel free to modify or use this software for any purpose, however if you re-distribute
// this software source you must include an acknowledgment of the original software and
// also distribute the source for free.
// TTFN,
//   Jon    www.TheValvePage.com
#include <stdio.h>
#include <string.h>

#include "defs.h"


// NOTE : Although the palette is read, it is not used. Assume
// an ordered palette for now.



// IMPORTANT : In Win32 (and presumably other 32-bit systems, WORDS may be
// stored as 32 bit instead of 16 bit. We need a WORD to occupy exactly two bytes.
// Using Visual C++ V6, the following #pragma forces the "correct" WORD sizing.
#pragma pack(2)


typedef
struct tagBITMAP_HEADER
{ 
    WORD   bfType ; // Should be characters 'BM'
    DWORD  bfSize; 
    WORD   bfReserved1; 
    WORD   bfReserved2; 
    DWORD  bfOffBits; 
    DWORD  biSize; 
    LONG   biWidth; 
    LONG   biHeight; 
    WORD   biPlanes; 
    WORD   biBitCount;
    DWORD  biCompression; 
    DWORD  biSizeImage; 
    LONG   biXPelsPerMeter; 
    LONG   biYPelsPerMeter; 
    DWORD  biClrUsed; 
    DWORD  biClrImportant; 

} BITMAP_HEADER ;

FILE *ip = NULL ;
BITMAP_HEADER bmhHeader ;
DWORD dwPalette[256] ;

BYTE OpenBitmapFile( char *filename )
{
    if( NULL != (ip=fopen( filename, "rb" )) )
    {
        fread( &bmhHeader, 1, sizeof(BITMAP_HEADER), ip ) ;

        /* First two bytes of file should be the characters 'BM' */

        if( bmhHeader.bfType != 0x4D42 )
        {
            printf("ERROR : File %s is not a valid bitmap\n", filename) ;
        }

        /* Ensure image is of correct size */

        else if( (bmhHeader.biWidth != PIX_PER_LINE) || (bmhHeader.biHeight != NUM_OF_LINES) )
        {
            printf("ERROR : File %s must be %d by %d pixels\n",
                           filename, PIX_PER_LINE, NUM_OF_LINES ) ;
        }

        /* Ensure using 256-colour palette */

        else if( bmhHeader.biBitCount != 8 ) // 8-bit
        {
            printf("ERROR : File %s must use an 8-bit palette", filename) ;
        }

        /* Read palette */
        /* TO DO : Handle colour palette */

        else
        {
            fread( dwPalette, 1, sizeof(dwPalette), ip ) ;
        }
    }

    return( (NULL != ip) ) ;
}



void CloseBitmapFile( void )
{
    if( NULL != ip )
    {
        fclose( ip ) ;
    }
}

#define IMAGE_START (sizeof(BITMAP_HEADER)+sizeof(dwPalette))

void ReadBitmapFile( WORD nLine, BYTE *pDest )
{
    // Remember, this is Micro$oft we're dealing with, so the
    // file's image will be upside-f*****g-down !

    nLine = (NUM_OF_LINES-1) - nLine ; // -1 cos nLine runs from 0 to (NUM_OF_LINES-1)

    // IMPORTANT : Following assumes PIX_PER_LINE is divisible by 4
    // since each line of data gets DWORD alligned.

    if( NULL != ip )
    {
        fseek( ip, (IMAGE_START + (nLine*PIX_PER_LINE)), SEEK_SET ) ;
        fread( pDest, 1, PIX_PER_LINE, ip ) ;
    }
}
